#!/usr/bin/env python
"""pikpak a python/gtk front end for ipkg"""

import gtk,GtkExtra,os,sys,popen2
from glob import glob
from string import split, strip, find, join

## Constants ##

VERSION = "0.5-2"

## All Possible field names (hopefully)

FLD_LU = { 'Package':0, 'Essential':1, 'Priority':2, 'Section':3,
           'Installed-Size':4, 'Debian-Maintainer':5, 'Maintainer':6,
           'Architecture':7, 'Version':8, 'Replaces':9, 'Pre-Depends':10,
           'Conflicts':11, 'Filename':12, 'Size':13, 'MD5sum':14,
           'Description':15, 'Depends':16, 'Origin':17, 'Bugs':18,
           'Provides':19, 'Suggests':20 }


## Edit this to change what fields are displayed and there widths

COLS = { 'Package':100, 'Version':50, 'Size':50, 'Section':50 }

##  Location of ipkg script

##IPKG_BIN = '/home/mallum/Projects/familiar/dist/ipkg/sh/ipkg'
IPKG_BIN = 'ipkg'

## Constants for menu / package actions 

IPKG_UPDATE      = 3
IPKG_INSTALL     = 1
IPKG_REMOVE      = 2
IPKG_UPGRADE     = 4
IPKG_UPGRADE_ALL = 5

## Constants for defining view types

V_ALL        = 0
V_INST       = 1
V_NOTINST    = 2

## Colors

UNINSTALLED_COLOR = 'orange red'
TO_UPGRADE_COLOR  = 'red'

## Interactive

INTERACTIVE = 1

## sections ( known )

SECTIONS = ( 'x11', 'games', 'web', 'net', 'editors',
             'graphics', 'comm', 'admin', 'devel', 'text', 'shells',
             'base', 'libs', 'kernel','non-US', 'extras', 'misc'  )

def getDiskSpace(mode=None):
    cmd_switch = ''
    if mode == 'nice': cmd_switch=' -h'
        
    for info in os.popen('df'+cmd_switch).readlines():
        parts = split(info)
        if strip(parts[5]) == '/':
            perc = parts[4]
            avail = parts[3]
            return (perc,avail)
    return None
    

class Pkgs:
    """Class for managing packages and states"""
    def __init__(self):
        ## Data structs for package info
        self.view    = []

        self.viewBy  = V_NOTINST # V_ALL
        self.sorted_by = 'Package'        
        self.section_filter = ['games', 'web', 'net', 'x11','misc',
                               'editors', 'graphics', 'comm', 'admin']

        ## init all 
        self.get_available()
        self.get_installed()
        self.update_view()

    def sort(self,bywhat,no_reverse=1):
        if FLD_LU.has_key(bywhat):
            if self.sorted_by == bywhat and no_reverse:
                self.view.reverse()
            else:
                self.view.sort(lambda s1,s2, bywhat=bywhat:
                               cmp(s1[FLD_LU[bywhat]] ,
                                   s2[FLD_LU[bywhat]] ) )
                self.sorted_by= bywhat


    def find(self, txt, lastFindIndex = 0):
        for p in range(lastFindIndex, len(self.view)):
            for part in self.view[p]:
                if find(str(part), txt) != -1:                    
                    return p
        return -1    

    def update_view(self, viewby = V_ALL):
        self.viewBy = viewby
        if self.viewBy == V_ALL:
            self.view = self.available[:]
        else:
        
            self.view = []
            for p in self.available:
                if self.viewBy == V_INST and \
                   self.is_installed(p[FLD_LU['Package']]):
                    self.view.append(p)
                elif self.viewBy == V_NOTINST and \
                     not self.is_installed(p[FLD_LU['Package']]):
                    self.view.append(p)


        if self.section_filter:
            i = 0
            while i < len(self.view):
                if self.view[i][FLD_LU['Section']] not in self.section_filter:
                    del self.view[i]
                else:
                    i = i + 1
        self.sort(self.sorted_by,0)

    def get_view(self):
        if self.viewBy == V_INST:
            return "Installed"
        elif self.viewBy == V_NOTINST:
            return "Uninstalled"
        else:
            return "All"
                
    def set_section_filter(self,val):
        if val is None:
            self.section_filter = []
        else:
            try: self.section_filter.index(val)
            ## section not listed
            except:
                self.section_filter.append(val)
                return
            self.section_filter.remove(val)


    def get_section_filter(self):
        return self.section_filter[:] 

    def get_field(self,row_num,col_name):
        if FLD_LU.has_key[col_name]:
            return self.view[row_num][FLD_LU[col_name]]
        else:
            return 0

    def is_installed(self,pkgname):
        return self.installed.has_key(pkgname)

    def get_installed_version(self,pkgname):
        if self.installed.has_key(pkgname):
            return self.installed[pkgname]
        else:
            return 0

    def get_available(self):
        ## Passes the output of ipkg info into a 2D indexed array
        try:                                      
            f = os.popen(IPKG_BIN + " info")
        except:
            print "ERROR :: Couldn't execute ipkg info! "
            print "         Check ipkg is in your path  "
            sys.exit()
            
        self.available = []
            
        pkg = [''] * 25                
        line = f.readline()
        while line:
            if line == "\n":                       ## Its a new Package entry
                if pkg[FLD_LU['Package']]:         ## its not just a blank line
                    self.available.append(pkg)
                pkg = [''] * 25
            else:
                parts = split(line[:-1],":")       ## remove newline
                if ':' in line and FLD_LU.has_key(parts[0]):
                    if parts[0] == 'Size':         ## Make size more readable
                        pkg[FLD_LU[parts[0]]] = int(strip(parts[1]))/1024
                    else:
                        pkg[FLD_LU[parts[0]]] = strip(parts[1])
                        
                else:                              ## Build description field
                    if pkg[FLD_LU["Description"]] == '.\n':
                        pkg[FLD_LU["Description"]] = \
                        pkg[FLD_LU["Description"]] + '\n\n'
                    else:
                        pkg[FLD_LU["Description"]] = \
                        pkg[FLD_LU["Description"]] + parts[0]

            line = f.readline()
	f.close()

    def get_installed(self):    ## Parses 'ipkg status' into a dictionary
        self.installed = {}     ## containing package -> version
        try:                                       
            f = os.popen(IPKG_BIN + " status","r")
        except:
            print "ERROR :: Couldn't execute ipkg status ! "
            print "         Check ipkg is in your path  "
            sys.exit()

        line = f.readline()
        while line:
            if line != '\n':
                p = split(line,":")
                if p[0] == 'Package':
                    current_pkg = strip(p[1])
                if p[0] == "Version":
                    self.installed[current_pkg] = strip(p[1])
            line = f.readline()                   
        f.close                                   

class PopUp(gtk.GtkWindow):
    def __init__(self,txt=''):
        gtk.GtkWindow.__init__(self,gtk.WINDOW_POPUP)
        self.set_position(gtk.WIN_POS_CENTER)
        self.connect("delete_event", self.delete_event)
        self.set_usize(180,100)
        self.set_position(gtk.WIN_POS_CENTER)
        self.lbl = gtk.GtkLabel(txt)
        self.lbl.set_justify(gtk.JUSTIFY_LEFT)
        self.lbl.show()
        self.lbl.set_line_wrap(gtk.TRUE)
        self.add(self.lbl)

    def delete_event(self, event=None):
        self.hide()
        return gtk.TRUE


class MsgBox(gtk.GtkDialog): ## simple sketchy dialog class
                             
    def __init__(self,master,lbl_text='',type=1,functions=None):
        gtk.GtkDialog.__init__(self)
        self.connect("delete_event", self.delete_event)
        self.top = master
        self.set_usize(180,150)
        self.set_position(gtk.WIN_POS_CENTER)
        ##self.set_uposition(20,80)
        if type != 3:
            lbl = gtk.GtkLabel(lbl_text)
            lbl.set_justify(gtk.JUSTIFY_RIGHT)
            lbl.show()
            lbl.set_line_wrap(gtk.TRUE)
            self.vbox.pack_start(lbl)
        if type == 1:                             ## ok box
            self.okay = gtk.GtkButton('Close')
            self.okay.show()
            self.action_area.pack_start(self.okay,expand=gtk.FALSE)
            if functions:
                self.okay.connect('clicked',functions[0])
            else:
                self.okay.connect('clicked',self.close_me)
        elif type == 2:                           ## find box
            self.findbutt = gtk.GtkButton('Find')
            self.findbutt.show()
            self.cancelbutt = gtk.GtkButton('Cancel')
            self.cancelbutt.show()
            self.action_area.pack_start(self.findbutt,expand=gtk.FALSE)
            self.action_area.pack_start(self.cancelbutt,expand=gtk.FALSE)
            self.entry = gtk.GtkEntry()
            self.entry.show()

            self.vbox.pack_start(self.entry)
            self.findbutt.connect('clicked',self.findCB)
            self.entry.connect('activate', self.findCB )
            self.cancelbutt.connect('clicked',self.close_me)
            self.entry.grab_focus()
        elif type == 3:                           ## large message
            self.closebutt = gtk.GtkButton('Close')
            self.closebutt.show()
            self.action_area.pack_start(self.closebutt,expand=gtk.FALSE)
            self.info = gtk.GtkScrolledWindow()
            self.gtkText = gtk.GtkText()
            self.gtkText.set_word_wrap( gtk.TRUE )
            self.info.add(self.gtkText)
            self.info.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            self.gtkText.set_point(0)
            self.gtkText.insert(None,None,None,lbl_text)
            adj = self.info.get_vadjustment() 
            adj.set_value(0.0) 
            self.gtkText.set_point(0)
            self.info.show_all() 
            self.err_lbl = gtk.GtkLabel("Ipkg Gave Errors ; ")
            self.err_lbl.set_justify(gtk.JUSTIFY_LEFT)
            self.err_lbl.show()
            self.vbox.pack_start(self.err_lbl)
            self.vbox.pack_end(self.info)
            self.closebutt.connect('clicked',self.close_me)

             
        self.vbox.show()
        self.action_area.show()
        self.set_modal(gtk.TRUE) 
        self.show()

    def findCB(self,btn):
        self.top.searchStr = self.entry.get_text()
        self.top.lastFindIndex = 0
        if self.top.doFind():
            self.close_me()

    def delete_event(win, event=None):
        self.hide()
        return gtk.TRUE

    def close_me(self,btn):
        self.hide()
        del self
        

class AppWindow(gtk.GtkWindow):         # Usual Base
    def __init__(self, title, width, height):
        gtk.GtkWindow.__init__(self)
        if title:
            self.set_title(title)
        self.set_usize(width,height)
        self.connect("delete_event", self.quit)

    def quit(self, *args):
        gtk.mainquit()

class Pikpak:

    def __init__(self):
        popup_win = PopUp('Please Wait.......\nReading Package Data')
        popup_win.show()
        while gtk.events_pending(): gtk.mainiteration()

        self.sec_checkItemCalled = 0

        self.pkgs = Pkgs()                ## init the pkgs object
        if gtk.screen_width() > 300:      ## Fit the display
            self.top = AppWindow("Pikpak", 320, 200)
        else:
            self.top = AppWindow("Pikpak", 240, 300)

        self.cmap = self.top.get_colormap() 

        self.vpane = gtk.GtkVPaned();   # to hold everything

        self.box = gtk.GtkVBox()
        self.top.add(self.box)
        self.box.show()
        self.init_menu()        

        self.box.pack_start(self.vpane, fill=gtk.TRUE, expand=gtk.TRUE)
        self.vpane.show()
        
        self.init_pkg_clist()
        self.init_info()
        self.init_status()

        self.selected = ""
        self.lastFindIndex = 0
        self.searchStr     = ''

        self.checkItemCalled = 0        ## Needed for ugly menu callback
        self.sec_checkItemCalled = 1

        self.itemf.get_widget('/View/Uninstalled' ).set_active(gtk.TRUE)

        ## set up view defults
        if self.pkgs.get_section_filter():
            secs = self.pkgs.get_section_filter()
            for sec in secs:
                path = '/View/By Section/' + sec
                self.itemf.get_widget( path ).set_active(gtk.TRUE)
        else:
            self.itemf.get_widget( '/View/By Section/All' ).set_active(gtk.TRUE)

        self.sec_checkItemCalled = 0 ## argggggggggg ! ugly workaround to
                                     ## stop callbacks
        self.update_pkg_list()



        popup_win.delete_event()

    def doFind(self):
        foundrow = self.pkgs.find(self.searchStr, self.lastFindIndex)
        if foundrow != -1:
            self.lastFindIndex = foundrow + 1
            self.clist.moveto( foundrow, 0, 0.5, 0.5 )
            self.clist.select_row( foundrow, 0)
            return gtk.TRUE
        else:
            MsgBox(self,"%s not found" % self.searchStr ).show()
            self.lastFindIndex = 0
            return gtk.FALSE

    def init_menu(self):
        ag = gtk.GtkAccelGroup()
        self.itemf = gtk.GtkItemFactory(gtk.GtkMenuBar, "<main>", ag)
        self.top.add_accel_group(ag)

        
        self.itemf.create_items([
            ('/File',             None, None, 0, '<Branch>'),
            ('/File/Update',      None, self.packageCB, IPKG_UPDATE, ''),
            ('/File/Upgrade (all)', None, self.packageCB, IPKG_UPGRADE_ALL, ''),
            ('/File/Exit',        None, self.top.quit, 0, ''),
            ('/Edit',             None, None, 0, '<Branch>'),
            ('/Edit/Find',        None, self.findCB, 1, ''),
            ('/Edit/Find Again',  None, self.findCB, 2, ''),
            ('/View',             None, None, 0, '<Branch>'),
            ('/View/All',         None, self.viewCB, V_ALL    , '<CheckItem>'),
            ('/View/Installed',   None, self.viewCB, V_INST   , '<CheckItem>'),
            ('/View/Uninstalled', None, self.viewCB, V_NOTINST, '<CheckItem>'),
            ('/View/By Section', None, None, 0, '<Branch>') ,
            ( "/View/sep1",     None, None, 0, "<Separator>" ),
            ('/View/Space', None, self.infoCB, 0, ''),
            ('/View/By Section/All',None,self.section_view_CB,-1,'<CheckItem>')
            ] )
        i = 0
        for s in SECTIONS:
                    self.itemf.create_items(
                        [ ('/View/By Section/' + s, None, self.section_view_CB, i, '<CheckItem>') ] )
                    i=i+1

        self.itemf.create_items([
	    ('/Package/Install',  None, self.packageCB, IPKG_INSTALL, ''),
            ('/Package/Remove',   None, self.packageCB, IPKG_REMOVE, ''),
            ('/Package/Upgrade',  None, self.packageCB, IPKG_UPGRADE, ''),
            ('/Help',             None, None, 0, '<Branch>'),
            ('/Help/About', None, self.aboutCB, 0, '')
            ])

        self.menubar = self.itemf.get_widget('<main>')
        self.box.pack_start(self.menubar, fill=gtk.FALSE, expand=gtk.FALSE)
        self.menubar.show()

        ## set up deaults

    def infoCB(self,action, widget):
        perc, avail = getDiskSpace(mode='nice')
        msg = "Space:\n\t %s Used\n%s Available" % \
              (perc, avail) 
        
        MsgBox(self.top,msg).show()

    def aboutCB(self,action, widget):
        msg = " PikPak %s \nBy Matthew Allum\n<mallum@handhelds.org>" % VERSION 
        MsgBox(self.top,msg).show()


    def section_view_CB(self,action, widget):
        if self.sec_checkItemCalled == 1:   ## Make sure set_active does not 
            return                          ## recall the callback
        self.sec_checkItemCalled = 1;
        if action == -1:
            self.pkgs.set_section_filter(None)
            for path in SECTIONS:
                path = '/View/By Section/' + path
                self.itemf.get_widget( path ).set_active(gtk.FALSE)
                self.itemf.get_widget( '/View/By Section/All' ).set_active(gtk.TRUE)
        else:
            self.pkgs.set_section_filter(SECTIONS[action])
            self.itemf.get_widget( '/View/By Section/All' ).set_active(gtk.FALSE)


        self.pkgs.update_view(self.pkgs.viewBy)
        self.update_pkg_list()
        self.sec_checkItemCalled = 0    
    
    def init_pkg_clist(self):
        self.scrollwin = gtk.GtkScrolledWindow()
        self.vpane.pack1( self.scrollwin, gtk.TRUE, gtk.TRUE)
        self.clist = gtk.GtkCList(len(COLS.keys()))
        self.clist.column_titles_show()
        x = 0
        for field_name in COLS.keys():
            self.clist.set_column_title(x,field_name)
            self.clist.set_column_width(x,COLS[field_name]);
            x = x + 1

        self.clist.row_is_visible(1)
        self.clist.connect("select_row" , self.selectRowCB)
        self.clist.connect("click_column", self.selectColumnCB)

        self.scrollwin.add(self.clist)
        self.scrollwin.show()
        self.clist.show()
        
    def init_info(self):
        self.info = gtk.GtkScrolledWindow()
        self.gtkText = gtk.GtkText()
        self.gtkText.set_word_wrap( gtk.TRUE )
        self.info.add(self.gtkText)
        self.info.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.vpane.pack2( self.info, gtk.TRUE, gtk.TRUE)
        self.info.show_all()    

    def init_status(self):
        self.status = gtk.GtkStatusbar()
        self.status.push(1,'ALL')
        self.box.pack_end(self.status, fill=gtk.FALSE, expand=gtk.FALSE)
        self.status.show()

    def set_status(self):
        msg = "%s: " % self.pkgs.get_view()[0:2]
        if msg == "Al: ": msg = "All: "
        sections = self.pkgs.section_filter[:]
        if sections:
            msg = msg + join(sections,',')  
        self.status.pop(1)
        self.status.push(1,msg)

    def viewCB(self,action,widget):
        if self.checkItemCalled == 1:   ## Make sure set_active does not 
            return                      ## recall the callback
        self.checkItemCalled = 1;

        ## More nasty workarounds for ItemFactory Radiobutton problems
        for path in [ '/View/All', '/View/Installed', '/View/Uninstalled' ]:
            if widget != self.itemf.get_widget( path ):
                self.itemf.get_widget( path ).set_active(gtk.FALSE)
        self.checkItemCalled = 0;
        
        self.pkgs.update_view(action)
        self.update_pkg_list()

    def findCB(self,action,widget):
        if action == 1:
            MsgBox(self,"Enter Search String",2).show()
        elif action == 2 and self.searchStr != '':
            self.doFind()

    def packageCB(self,action,widget):
        if not self.selected and action != IPKG_UPDATE and \
           action != IPKG_UPGRADE_ALL:
            MsgBox(self.top,"No Package Selected!").show()
            return

        if action == IPKG_INSTALL: # check for disk space
            p,a = getDiskSpace();
            ##print "%s vs %s" % (self.space_req, a)
            if int(self.space_req) > int(a):
                MsgBox(self.top,"Not enough space left to install.").show()
                return
            
        popup_win = None
        spin = ( '|', '/', '-', '\\' )
        spin_idx = 0
        ext_cmd = 'rxvt -e '
        xtra_swt = ''
        msg = ''
        if not INTERACTIVE:
            ext_cmd = ''
            xtra_swt ='  -force-defaults'
        
        if action == IPKG_INSTALL:   
            cmd_str = ext_cmd + IPKG_BIN +xtra_swt+" install " + self.selected
            msg = 'Installing '
        elif action == IPKG_REMOVE:
            cmd_str = ext_cmd + IPKG_BIN + xtra_swt+" remove " + self.selected
            msg = 'Removing '
        elif action == IPKG_UPDATE:
            cmd_str = ext_cmd + IPKG_BIN + xtra_swt + " update "
            msg = 'Updateing '
        elif action == IPKG_UPGRADE:
            cmd_str = ext_cmd + IPKG_BIN + xtra_swt+" upgrade " + self.selected
            msg = 'Upgrading '
        elif action == IPKG_UPGRADE_ALL:
            msg = 'Upgrading '
            cmd_str = ext_cmd + IPKG_BIN + xtra_swt + " upgrade " 

        if not INTERACTIVE:
            popup_win = PopUp(msg + "\n" + spin[spin_idx])
            popup_win.show()
            stderr_arr = []
            o = popen2.Popen4(cmd_str) # capture stderr
            ## hopefully make stderr non-blocking
            poll_state = o.poll() 
            while poll_state == -1:
                while gtk.events_pending(): gtk.mainiteration()
                stderr_arr.append(o.fromchild.readline())
                while gtk.events_pending(): gtk.mainiteration()
                popup_win.lbl.set_text(msg + "\n" + spin[spin_idx])
                spin_idx = spin_idx + 1
                if spin_idx == 4: spin_idx = 0
                poll_state = o.poll() 

        else:
            os.system(cmd_str)
            
        

        while gtk.events_pending(): gtk.mainiteration()

        if popup_win:
            popup_win.lbl.set_text('Please Wait.......\nUpdating Pacakge Data')
        else:
            popup_win = PopUp('Please Wait.......\nUpdating Package Data')
            popup_win.show()

        while gtk.events_pending(): gtk.mainiteration()
            
        if action == IPKG_UPDATE:
            self.pkgs.get_available();
            
        self.pkgs.get_installed();
        self.update_pkg_list()
        popup_win.delete_event()

        if poll_state != 0:
            MsgBox(self.top,lbl_text='\n'.join(stderr_arr),type=3).show()
        
 
    def selectColumnCB(self, *args):
        tosort = int(args[1])
        self.clist.clear()
        self.pkgs.sort(COLS.keys()[tosort])
        self.update_pkg_list()

    # TODO: Make class methods of all the nasty pkg lookupup
    #       stuff in the methods below
            
    def selectRowCB(self, *args):
        self.selected = self.pkgs.view[int(args[1])][FLD_LU["Package"]]
        self.space_req = self.pkgs.view[int(args[1])][FLD_LU["Size"]]
        self.gtkText.set_point(self.gtkText.get_length())
        self.gtkText.backward_delete(self.gtkText.get_length())
        self.gtkText.set_point(0)
        self.gtkText.insert(None,None,None,
                            self.pkgs.view[int(args[1])][FLD_LU["Description"]])
        extra = '\n\n'
        for fld in FLD_LU.keys():
            if not COLS.has_key(fld) and fld != 'Description':
                if self.pkgs.view[int(args[1])][FLD_LU[fld]]:
                    extra = extra + fld + ": " + self.pkgs.view[int(args[1])][FLD_LU[fld]] + '\n'
        self.gtkText.insert(None,None,None, extra)
            
        adj = self.info.get_vadjustment() 
        adj.set_value(0.0) 
        self.gtkText.set_point(0)

    def update_pkg_list(self):
        rowidx = 0
        self.clist.clear()
#       icol  = self.cmap.alloc(51400,51400,51400);
        icol  = self.cmap.alloc(UNINSTALLED_COLOR);
#       incol = self.cmap.alloc(26200,26200,26200);
        incol = self.cmap.alloc(TO_UPGRADE_COLOR);
        for p in self.pkgs.view:
            cols = []
            for f in COLS.keys():
                cols.append(str(p[FLD_LU[f]]))
            self.clist.append( cols )    
            if self.pkgs.is_installed(p[FLD_LU["Package"]]):
                if self.pkgs.get_installed_version(p[FLD_LU["Package"]]) != p[FLD_LU["Version"]]:
                    self.clist.set_background(rowidx, incol )
                    ## color for installed but newer available
                else:
                    self.clist.set_background(rowidx, icol )
                    ## color for installed package
            rowidx = rowidx + 1
        
        self.set_status()
        
    def mainloop(self):
        self.top.show()
        gtk.mainloop()

def load_rc():
    global COLS, UNINSTALLED_COLOR, TO_UPGRADE_COLOR,INTERACTIVE
    try:
        rcfile = open(os.environ['HOME'] + "/.pikpak",'r')
    except:
        # write a default file here
        try:
            print "Atempting to create %s" % os.environ['HOME'] + "/.pikpak"
            rcfile = open(os.environ['HOME'] + "/.pikpak",'w')
        except:
            print "ERROR: could not write %s" % os.environ['HOME'] + "/.pikpak"
            sys.exit(0)

        rcfile.writelines( 
            [ '## This is the deualt pikpak configuration file\n',
            '\n',
            '## Define the ipkg attributes to show as columns\n',
            '## Togeather with there widths in pixels\n',
            '\n',
            'COLUMNS = Package:100,Version:50,Size:50,Section:50\n',
            '\n',
            '## Define the colors for upradable and uninstalled packages\n'
            '## See /usr/X11R6/lib/x11/rgb.txt for valid definitions\n',
            '\n',
            'UNINSTALLED_COLOR = orange red\n',
            'TO_UPGRADE_COLOR  = red     \n',
            '\n',
            '## Do you want pikpak to launch ipkg interactively or not\n',
            '## ie, launch a term with ipkg running in it\n',
            '\n',
            'IPKG_INTERACTIVE = NO\n'
            ] )
        rcfile.close()
        return
            
    line = rcfile.readline()
    while line:
        if line[0] == '#' or line[0] == '\n':
            line = rcfile.readline()
            continue
        key,val = split(line,'=')
        key = strip(key)
        val = strip(val)
        if key == 'COLUMNS':
            COLS = {}
            cols = split(val,',')
            if not cols: cols = val ## fix for only 1 column
            for pair in cols:
                n,w = split(pair,':')
                COLS[n] = int(w)
        elif key == 'UNINSTALLED_COLOR':
            UNINSTALLED_COLOR = val
        elif key == 'TO_UPGRADE_COLOR':
            TO_UPGRADE_COLOR  = val
        elif key == 'IPKG_INTERACTIVE':
            if val == 'YES':
                INTERACTIVE = 1
            else:
                INTERACTIVE = 0
        line = rcfile.readline()

def main():
    load_rc()
    s = Pikpak()
    ##s.__init__(self)
    s.mainloop()
if __name__ == "__main__":
    main()

















